@@ -6,22 +6,17 @@ __metaclass__ = type
 
 import os
 import os.path
-import pkgutil
 import re
 import sys
 
 from types import ModuleType
 
 from ansible.module_utils._text import to_bytes, to_native, to_text
+from ansible.module_utils.compat.importlib import import_module
 from ansible.module_utils.six import iteritems, string_types, with_metaclass
+from ansible.utils.path import cs_open
 from ansible.utils.singleton import Singleton
 
-# HACK: keep Python 2.6 controller tests happy in CI until they're properly split
-try:
-    from importlib import import_module
-except ImportError:
-    import_module = __import__
-
 _SYNTHETIC_PACKAGES = {
     # these provide fallback package definitions when there are no on-disk paths
     'ansible_collections': dict(type='pkg_only', allow_external_subpackages=True),
@@ -269,7 +264,7 @@ class AnsibleCollectionLoader(with_metaclass(Singleton, object)):
         return os.path.join(path, ns_path_add)
 
     def get_data(self, filename):
-        with open(filename, 'rb') as fd:
+        with cs_open(filename, 'rb') as fd:
             return fd.read()
 
 
