@@ -14,6 +14,7 @@ import pytest
 
 from ansible.module_utils._text import to_native
 from ansible.module_utils.six import PY2
+from ansible.module_utils.compat import selectors
 
 
 class OpenBytesIO(BytesIO):
@@ -28,9 +29,6 @@ class OpenBytesIO(BytesIO):
 
 @pytest.fixture
 def mock_os(mocker):
-    def mock_os_read(fd, nbytes):
-        return os._cmd_out[fd].read(nbytes)
-
     def mock_os_chdir(path):
         if path == '/inaccessible':
             raise OSError(errno.EPERM, "Permission denied: '/inaccessible'")
@@ -42,11 +40,6 @@ def mock_os(mocker):
             return os.getcwd.return_value + '/' + path
 
     os = mocker.patch('ansible.module_utils.basic.os')
-    os._cmd_out = {
-        # os.read() is returning 'bytes', not strings
-        mocker.sentinel.stdout: BytesIO(),
-        mocker.sentinel.stderr: BytesIO(),
-    }
 
     os.path.expandvars.side_effect = lambda x: x
     os.path.expanduser.side_effect = lambda x: x
@@ -54,26 +47,79 @@ def mock_os(mocker):
     os.getcwd.return_value = '/home/foo'
     os.path.isdir.return_value = True
     os.chdir.side_effect = mock_os_chdir
-    os.read.side_effect = mock_os_read
     os.path.abspath.side_effect = mock_os_abspath
 
     yield os
 
 
+class DummyFileObj():
+    def __init__(self, fileobj):
+        self.fileobj = fileobj
+
+
+class SpecialBytesIO(BytesIO):
+    def __init__(self, *args, **kwargs):
+        fh = kwargs.pop('fh', None)
+        super(SpecialBytesIO, self).__init__(*args, **kwargs)
+        self.fh = fh
+
+    def fileno(self):
+        return self.fh
+
+    # We need to do this because some of our tests create a new value for stdout and stderr
+    # The new value is able to affect the string that is returned by the subprocess stdout and
+    # stderr but by the time the test gets it, it is too late to change the SpecialBytesIO that
+    # subprocess.Popen returns for stdout and stderr.  If we could figure out how to change those as
+    # well, then we wouldn't need this.
+    def __eq__(self, other):
+        if id(self) == id(other) or self.fh == other.fileno():
+            return True
+        return False
+
+
+class DummyKey:
+    def __init__(self, fileobj):
+        self.fileobj = fileobj
+
+
 @pytest.fixture
 def mock_subprocess(mocker):
-    def mock_select(rlist, wlist, xlist, timeout=1):
-        return (rlist, [], [])
 
-    fake_select = mocker.patch('ansible.module_utils.basic.select')
-    fake_select.select.side_effect = mock_select
+    class MockSelector(selectors.BaseSelector):
+        def __init__(self):
+            super(MockSelector, self).__init__()
+            self._file_objs = []
+
+        def register(self, fileobj, events, data=None):
+            self._file_objs.append(fileobj)
+
+        def unregister(self, fileobj):
+            self._file_objs.remove(fileobj)
+
+        def select(self, timeout=None):
+            ready = []
+            for file_obj in self._file_objs:
+                ready.append((DummyKey(subprocess._output[file_obj.fileno()]), selectors.EVENT_READ))
+            return ready
+
+        def get_map(self):
+            return self._file_objs
+
+        def close(self):
+            super(MockSelector, self).close()
+            self._file_objs = []
+
+    selectors.DefaultSelector = MockSelector
 
     subprocess = mocker.patch('ansible.module_utils.basic.subprocess')
+    subprocess._output = {mocker.sentinel.stdout: SpecialBytesIO(b'', fh=mocker.sentinel.stdout),
+                          mocker.sentinel.stderr: SpecialBytesIO(b'', fh=mocker.sentinel.stderr)}
+
     cmd = mocker.MagicMock()
     cmd.returncode = 0
     cmd.stdin = OpenBytesIO()
-    cmd.stdout.fileno.return_value = mocker.sentinel.stdout
-    cmd.stderr.fileno.return_value = mocker.sentinel.stderr
+    cmd.stdout = subprocess._output[mocker.sentinel.stdout]
+    cmd.stderr = subprocess._output[mocker.sentinel.stderr]
     subprocess.Popen.return_value = cmd
 
     yield subprocess
@@ -84,7 +130,6 @@ def rc_am(mocker, am, mock_os, mock_subprocess):
     am.fail_json = mocker.MagicMock(side_effect=SystemExit)
     am._os = mock_os
     am._subprocess = mock_subprocess
-    am.get_buffer_size = mocker.MagicMock(return_value=900)
     yield am
 
 
@@ -151,7 +196,11 @@ class TestRunCommandPrompt:
 
     @pytest.mark.parametrize('stdin', [{}], indirect=['stdin'])
     def test_prompt_match_wo_data(self, mocker, rc_am):
-        rc_am._os._cmd_out[mocker.sentinel.stdout] = BytesIO(b'Authentication required!\nEnter password: ')
+        rc_am._subprocess._output = {mocker.sentinel.stdout:
+                                     SpecialBytesIO(b'Authentication required!\nEnter password: ',
+                                                    fh=mocker.sentinel.stdout),
+                                     mocker.sentinel.stderr:
+                                     SpecialBytesIO(b'', fh=mocker.sentinel.stderr)}
         (rc, _, _) = rc_am.run_command('foo', prompt_regex=r'[pP]assword:', data=None)
         assert rc == 257
 
@@ -181,7 +230,10 @@ class TestRunCommandOutput:
 
     @pytest.mark.parametrize('stdin', [{}], indirect=['stdin'])
     def test_ascii_stdout(self, mocker, rc_am):
-        rc_am._os._cmd_out[mocker.sentinel.stdout] = BytesIO(b'hello')
+        rc_am._subprocess._output = {mocker.sentinel.stdout:
+                                     SpecialBytesIO(b'hello', fh=mocker.sentinel.stdout),
+                                     mocker.sentinel.stderr:
+                                     SpecialBytesIO(b'', fh=mocker.sentinel.stderr)}
         (rc, stdout, stderr) = rc_am.run_command('/bin/cat hello.txt')
         assert rc == 0
         # module_utils function.  On py3 it returns text and py2 it returns
@@ -190,8 +242,12 @@ class TestRunCommandOutput:
 
     @pytest.mark.parametrize('stdin', [{}], indirect=['stdin'])
     def test_utf8_output(self, mocker, rc_am):
-        rc_am._os._cmd_out[mocker.sentinel.stdout] = BytesIO(u'Žarn§'.encode('utf-8'))
-        rc_am._os._cmd_out[mocker.sentinel.stderr] = BytesIO(u'لرئيسية'.encode('utf-8'))
+        rc_am._subprocess._output = {mocker.sentinel.stdout:
+                                     SpecialBytesIO(u'Žarn§'.encode('utf-8'),
+                                                    fh=mocker.sentinel.stdout),
+                                     mocker.sentinel.stderr:
+                                     SpecialBytesIO(u'لرئيسية'.encode('utf-8'),
+                                                    fh=mocker.sentinel.stderr)}
         (rc, stdout, stderr) = rc_am.run_command('/bin/something_ugly')
         assert rc == 0
         # module_utils function.  On py3 it returns text and py2 it returns
