@@ -17,7 +17,7 @@
 # along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-from ansible.compat.tests.mock import patch, Mock, MagicMock, call
+from units.compat.mock import patch, Mock, MagicMock, call
 from units.modules.utils import set_module_args
 from .netscaler_module import TestModule, nitro_base_patcher
 
